% script file balken.m
% deformation of a rod

clc; clear all; close all;

n = 41;
x = linspace(0,1,n)';
h = x(2)-x(1);

K = 1; % normed elasticity coefficient

% build the system
md = ones(n,1);
A = spdiags([md -4*md 6*md -4*md md],[-2,-1,0,1,2],n,n);

% force
%f = -100*x.^12;
f = zeros(n,1);
source = (find((x>0.3)&(x<0.4)));
f(source) = -1;


% rhs of the system: 
rhs = (h^4)*f/K;

% boundary values:
rhs(1) = 0;
rhs(2) = 0;
rhs(n-1) = 0;
rhs(n) = 0;

% rewrite the boundary points
A(1,1) = 1; A(1,2) = 0; A(1,3) = 0;
A(2,1) = 0; A(2,2) = 1; A(2,3) = 0; A(2,4) = 0;
A(n-1,n-3) = 0; A(n-1,n-2) = 0; A(n-1,n-1) = 1; A(n-1,n) = 0;
A(n,n-2) = 0; A(n,n-1) = 0; A(n,n) = 1;

disp('slash')
tic
u = A\rhs;
toc
disp(' ');


figure(1);
subplot(2,1,1)
plot(x,f,'r');
title('Force');

subplot(2,1,2);
plot(x,u,'b');
title('Deformation');



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

x0 = 0.00*rand(n,1);
x0 = 0.001*ones(n,1);
%x0 = rhs;
maxit = 50000;
toler = 1e-6;


% to apply minres, make the system matrix symmetric
% so that application is valid. 
% eliminate boundary points
md = ones(n-4,1);
A1 = spdiags([md -4*md 6*md -4*md md],[-2,-1,0,1,2],n-4,n-4);

% move them to the rhs - nothing has to be done because of homogeneity
rhs1 = (h^4)*f(3:n-2)/K;

% make matrix indefinite
%A1(3,:)=-A1(3,:);
%rhs1(3)=-rhs(3);


% compare the runtimes and iteration counts
disp('minres')
tic
uMR = minres_2(A1,rhs1,x0(3:n-2),maxit,toler);
toc

disp(' ')
disp('cg')
tic
uMR = cg(A1,rhs1,x0(3:n-2),toler,maxit);
toc


maxit = 5000;
disp(' ')
disp('GMRES')
tic
uMR = Gmres(A1,rhs1,toler,maxit,x0(3:n-2));
toc

disp(' ');
disp(' ');

%return
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% visualise iterations

maxit = 100;

% run the iteration and store the iterates

[U_MIN,Rnorm_MIN] = minres_store(A1,rhs1,x0(3:n-2),maxit,toler);
% add zeros
U_MIN = [zeros(2,maxit+1); U_MIN; zeros(2,maxit+1)];

[U_CG,Rnorm_CG] = cg_store(A1,rhs1,x0(3:n-2),toler,maxit);
% add zeros
U_CG = [zeros(2,maxit+1); U_CG; zeros(2,maxit+1)];


[U_GMR,Rnorm_GMR] = Gmres_store(A1,rhs1,toler,maxit,x0(3:n-2));
U_GMR = [zeros(2,maxit+1); U_GMR; zeros(2,maxit+1)];

normb = norm(rhs);
Rnorm_GMR = (1/normb)*Rnorm_GMR;
Rnorm_MIN = (1/normb)*Rnorm_MIN;
Rnorm_CG  = (1/normb)*Rnorm_CG;



% visualise 
for i=1:maxit+1
  figure(2);
    
  
  subplot(2,3,1);
  plot(x,u,'b',x,U_GMR(:,i),'g')
  title(['GMRES: ',num2str(i-1)]);
  
  subplot(2,3,4);
  loglog(1:i,Rnorm_GMR(1:i),'r.-');
  axis([1 maxit 0 max(Rnorm_GMR)]);  
  xlabel('x');
  ylabel('||r||');

  
  subplot(2,3,2);
  plot(x,u,'b',x,U_MIN(:,i),'g')
  title(['MINRES: ',num2str(i-1)]);
  
  subplot(2,3,5);
  loglog(1:i,Rnorm_MIN(1:i),'r.-');
  axis([1 maxit 0 max(Rnorm_MIN)]);  
  xlabel('x');
  
  
  subplot(2,3,3);
  plot(x,u,'b',x,U_CG(:,i),'g')
  title(['CG: ',num2str(i-1)]);
  
  subplot(2,3,6);
  loglog(1:i,Rnorm_CG(1:i),'r.-');
  axis([1 maxit 0 max(Rnorm_CG)]);  
  xlabel('x')
  
  
  
  drawnow
  %waitforbuttonpress
  %pause(0.1);
  
end


