function [X,Resnorm] = minres_store(A,b,x0,maxit,toler);
% minres algorithm
% implementation from Kanzow
% store all iterates

r0=b-A*x0; zeta0=norm(r0); zeta=zeta0; v=0; v_new=r0/zeta0; beta_new=0;
c=1; s=0; c_new=1; s_new=0;
p=0; p_new=0; x=x0;

tolb = norm(b)*toler;

dim = length(x0);
X = zeros(dim,maxit+1);
X(:,1) = x0;

Resnorm = zeros(maxit+1,1);
Resnorm(1) = zeta;

for k=1:maxit
  % Lanczos
  beta=beta_new;
  v_old=v; v=v_new;
  v_new=A*v-beta*v_old;
  alpha=v_new'*v;
  v_new=v_new-alpha*v;
  beta_new=norm(v_new);
  v_new=v_new/beta_new;
  
  % Givens
  c_old=c; s_old=s; c=c_new; s=s_new;
  rho1=s_old*beta;
  rho2=c*c_old*beta+s*alpha;
  rho3_tilde=c*alpha-s*c_old*beta;
  
  tau = abs(rho3_tilde) + abs(beta_new);
  nu = tau*sqrt((rho3_tilde/tau)^2 + (beta_new/tau)^2);
  c_new=rho3_tilde/nu;
  s_new=beta_new/nu;
  
  rho3=nu;
  
  % Aufdatierung von Pk
  p_old=p; p=p_new;
  p_new=(v-rho1*p_old-rho2*p)/rho3;
  
  % Update
  x=x+c_new*zeta*p_new;
  
  X(:,k+1)=x;
  
  % Abbruchskriterium
  zeta=-s_new*zeta;
  
  
  Resnorm(k+1)=abs(zeta);
  if abs(zeta)<=tolb
    X(:,[k+1:maxit+1]) = repmat(x,[1 maxit-k+1]);
    Resnorm(k+1:maxit+1) = norm(b-A*x);
    break;
  end
end

if k<maxit
 disp(['converged at ',num2str(k),'-th iteration']);
else
 warning('maximal number of iterations reached')
end